// ---------------------------------------------------------------------------
//      
//
//   :
//
//   LimitForShowLabel -    
//   MinimalLineLevel  -     
//   MaximalLineLevel  -     
//   OptimizeZigZag    -   
//                       (     )
//   ColorMaximum      -   
//   ColorMinimum      -   
//   ColorLines        -   
//   OffsetForMaximums -      
//   OffsetForMinimums -      
//
//   2 -       
//   3 -       
//              -        . 
//                 -  
//               .
//                 - .. ,  
//                 -  , 
//                     
//             ,    -     (
//              ).
//   4 -       " ,
//              ".     ,
//                  .
//   5 -   ,    
//               .    .
//                .
//   6 -       . -
//               ""  .
// ---------------------------------------------------------------------------

#property show_inputs

#define PREFIX_TEXT "t_"
#define PREFIX_LINE "l_"
#define FONT_SIZE    10

extern int   LimitForShowLabel = 1;
extern int   MinimalLineLevel  = 1;
extern int   MaximalLineLevel  = 1;
extern bool  OptimizeZigZag    = true;
extern color ColorMaximum      = C'0,255,0';
extern color ColorMinimum      = C'255,0,0';
extern color ColorLines        = C'255,255,255';
extern int   OffsetForMaximums = 150;
extern int   OffsetForMinimums = 10;

int maximum[], minimum[];

// 
int init() {
  //    
  ArrayResize(maximum, Bars);
  ArrayResize(minimum, Bars);
  
  ArrayInitialize(maximum, 0);
  ArrayInitialize(minimum, 0);
  
  //   
  ObjectsDeleteAll(0, OBJ_TEXT);
  ObjectsDeleteAll(0, OBJ_TREND);

  return(0);
}

//  
int start() {
  int i = 1;
  
  while(pass(i)) i++;
  
  draw_result(i - 1);

  return(0);
}

//   
//  true,      
bool pass(int level) {
  return(pass_maximum(level) || pass_minimum(level));
}

//      
//       
double find_left_maximum(int bar) {
  int i, level = maximum[bar];
  
  for(i = bar + 1; i < Bars; i++) {
    if(maximum[i] >= level) {
      return(High[i]);
    }
  }

  //    
  return(High[bar] + 1);
}

//      
//       
double find_right_maximum(int bar) {
  int i, level = maximum[bar];
  
  for(i = bar - 1; i >= 0; i--) {
    if(maximum[i] >= level) {
      return(High[i]);
    }
  }

  //    
  return(High[bar] + 1);
}

//     
//  true,    
bool pass_maximum(int level) {
  int i, limit = Bars - 2, prev = level - 1, result = false;
  
  for(i = 1; i < limit; i++) {
    if(maximum[i] == prev) {
      double l = find_left_maximum(i);
      double r = find_right_maximum(i);
    
      if((l <= High[i] && r < High[i]) || (l < High[i] && r <= High[i])) {
        maximum[i] = level;
        result     = true;
      }
    }
  }

  return(result);
}

//      
//       
double find_left_minimum(int bar) {
  int i, level = minimum[bar];
  
  for(i = bar + 1; i < Bars; i++) {
    if(minimum[i] >= level) {
      return(Low[i]);
    }
  }

  //    
  return(Low[bar] - 1);
}

//      
//       
double find_right_minimum(int bar) {
  int i, level = minimum[bar];
  
  for(i = bar - 1; i >= 0; i--) {
    if(minimum[i] >= level) {
      return(Low[i]);
    }
  }

  //    
  return(Low[bar] - 1);
}

//     
//  true,    
bool pass_minimum(int level) {
  int i, limit = Bars - 2, prev = level - 1, result = false;
  
  for(i = 1; i < limit; i++) {
    if(minimum[i] == prev) {
      double l = find_left_minimum(i);
      double r = find_right_minimum(i);
    
      if((l >= Low[i] && r > Low[i]) || (l > Low[i] && r >= Low[i])) {
        minimum[i] = level;
        result     = true;
      }
    }
  }

  return(result);
}

//  
void optimize_minimums(int bar, int level) {
  int i;
  
  for(i = bar + 1; i < Bars; i++) {
    if(minimum[i] >= level) {
      if(Low[i] < Low[bar]) {
        minimum[bar] = 0;
      } else {
        minimum[i]   = 0;
      }
      
      break;
    }
  }
}

//  
void optimize_maximums(int bar, int level) {
  int i;
  
  for(i = bar + 1; i < Bars; i++) {
    if(maximum[i] >= level) {
      if(High[i] > High[bar]) {
        maximum[bar] = 0;
      } else {
        maximum[i]   = 0;
      }
      
      break;
    }
  }
}

//     
void optimize_level(int level) {
  int i, j = 0;
    
  for(i = Bars - 1; i >= 0; i--) {
    if(maximum[i] >= level) {
      if(j == 1) {
        optimize_maximums(i, level);
      } else {
        j = 1;
      }
    }
      
    if(minimum[i] >= level) {
      if(j == -1) {
        optimize_minimums(i, level);
      } else {
        j = -1;
      }
    }
  }
}

//   
void build_line(int level, int bar_to, 
  datetime time_from, double price_from, datetime time_to, double price_to) {

  string n = StringConcatenate(PREFIX_LINE, level, "_", bar_to);

  ObjectCreate(n, OBJ_TREND, 0, time_from, price_from, time_to, price_to);
  ObjectSet(n, OBJPROP_COLOR, ColorLines);
  ObjectSet(n, OBJPROP_RAY,   0);  
}

//  
//   -    
void draw_result(int last_level) {
  int      i, j, k;
  string   n;
  double   prev_price;
  datetime prev_time;
  
  //   
  for(i = 0; i < Bars; i++) {
    if(maximum[i] >= LimitForShowLabel) {
      n = StringConcatenate(PREFIX_TEXT, i, "_max_", maximum[i]);
      
      ObjectCreate(n, OBJ_TEXT, 0, Time[i], High[i] + OffsetForMaximums * Point);
      ObjectSet(n, OBJPROP_COLOR, ColorMaximum);
      ObjectSetText(n, "" + maximum[i], FONT_SIZE);
    }

    if(minimum[i] >= LimitForShowLabel) {
      n = StringConcatenate(PREFIX_TEXT, i, "_min_", minimum[i]);
      
      ObjectCreate(n, OBJ_TEXT, 0, Time[i], Low[i] - OffsetForMinimums * Point);
      ObjectSet(n, OBJPROP_COLOR, ColorMinimum);
      ObjectSetText(n, "" + minimum[i], FONT_SIZE);
    }
  }
  
  //    
  if(last_level > MaximalLineLevel) {
   k = MaximalLineLevel;
  } else {
   k = last_level;
  }
  
  //   
  for(i = MinimalLineLevel; i <= k; i++) {
    if(OptimizeZigZag) {
      optimize_level(i);
    }
  
    prev_price = -1;
  
    for(j = Bars - 1; j >= 0; j--) {
      if(maximum[j] >= i) {
        if(prev_price > 0) {
          build_line(i, j, prev_time, prev_price, Time[j], High[j]);
        }
        
        prev_time  = Time[j];        
        prev_price = High[j];
      }

      if(minimum[j] >= i) {
        if(prev_price > 0) {
          build_line(i, j, prev_time, prev_price, Time[j], Low[j]);
        }
        
        prev_time  = Time[j];        
        prev_price = Low[j];
      }
    }
  }
}


